﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace RFClassGraph
{
    public partial class MainWindow : Window
    {
        DockPanel Panel = new DockPanel();

        public MainWindow()
        {
            InitializeComponent();
            this.Content = Panel;
            Loaded += MainWindow_Loaded;
        }

        private void MainWindow_Loaded(object sender, RoutedEventArgs e)
        {
            try
            {
                Microsoft.Msagl.WpfGraphControl.GraphViewer graphViewer = new Microsoft.Msagl.WpfGraphControl.GraphViewer();
                graphViewer.BindToPanel(Panel);
                graph = new Microsoft.Msagl.Drawing.Graph();
                if (Environment.GetCommandLineArgs().Length == 2)
                    LoadFlowData(Environment.GetCommandLineArgs()[1]);
                graph.Attr.LayerDirection = Microsoft.Msagl.Drawing.LayerDirection.LR;
                graphViewer.Graph = graph;
            }
            catch (System.Exception ex)
            {
                System.Windows.MessageBox.Show(ex.ToString());
            }
        }

        private void LoadFlowData(string path)
        {
            RuntimeFlowExt.FlowData flow = RF.Utils.Load(path);
            functions = new System.Collections.Generic.Dictionary<uint, RuntimeFlowExt.FunctionInfo>();
            if (flow.summary != null && flow.summary.functions != null)
            {
                foreach (RuntimeFlowExt.KeyValuePair<uint, RuntimeFlowExt.FunctionInfo> i in flow.summary.functions)
                    functions[i.Key] = i.Value;
            }

            foreach (RuntimeFlowExt.ThreadNode t in flow.flow)
            {
                foreach (RuntimeFlowExt.FlowNode f in t.children)
                    LoadFlowData(f);
            }
        }

        private void LoadFlowData(RuntimeFlowExt.FlowNode node)
        {
            string caller = FindClassName(node);
            foreach (RuntimeFlowExt.FlowNode f in node.children)
            {
                string callee = FindClassName(f);
                if (!string.IsNullOrEmpty(caller) && !string.IsNullOrEmpty(callee) && caller != callee && !FindEdge(caller, callee))
                    graph.AddEdge(caller, callee);
                LoadFlowData(f);
            }
        }

        private bool FindEdge(string caller, string callee)
        {
            foreach (var i in graph.Edges)
            {
                if (i.Source == caller && i.Target == callee)
                    return true;
            }
            return false;
        }

        private string FindClassName(RuntimeFlowExt.FlowNode node)
        {
            return functions[node.functionID].className;
        }

        private Microsoft.Msagl.Drawing.Graph graph;
        private System.Collections.Generic.Dictionary<uint, RuntimeFlowExt.FunctionInfo> functions;
    }
}
