﻿namespace RFClassGraphExt
{
    class Extension : RuntimeFlowExt.IExtension
    {
        public Extension()
        {
        }

        public void Init(RuntimeFlowExt.IEngine engine_)
        {
            resolver = new SatelliteAssembliesResolver();
            window = new MainWindow();
            window.Show();

            threads = new System.Collections.Generic.Dictionary<uint, Thread>();
            engine = engine_;
            engine.FunctionEnter += OnFunctionEnter;
            engine.FunctionLeave += OnFunctionLeave;
            engine.FunctionsReset += OnFunctionsReset;
            engine.Clear += OnClear;
        }

        public void Close()
        {
            engine.FunctionEnter -= OnFunctionEnter;
            engine.FunctionLeave -= OnFunctionLeave;
            engine.FunctionsReset -= OnFunctionsReset;
            engine.Clear -= OnClear;
            window.Close();
            resolver.Close();
        }

        private void OnFunctionEnter(uint threadID, uint functionID, uint paramsID)
        {
            if (!threads.ContainsKey(threadID))
                threads.Add(threadID, new Thread());

            Thread thread = threads[threadID];
            uint? currentFunction = thread.GetCurrentFunction();
            if (currentFunction.HasValue)
                AddEdge(currentFunction.Value, functionID);
            thread.Enter(functionID);
        }

        private void OnFunctionLeave(uint threadID, uint functionID, uint returnValueID)
        {
            threads[threadID].Leave(functionID);
        }

        private void OnFunctionsReset()
        {
            threads.Clear();
            window.Clear();
        }

        private void OnClear()
        {
            window.Clear();
        }

        private void AddEdge(uint sourceFunctionID, uint targetFunctionID)
        {
            string source = FindClassName(sourceFunctionID);
            string target = FindClassName(targetFunctionID);
            if (!string.IsNullOrEmpty(source) && !string.IsNullOrEmpty(target) && source != target && !window.FindEdge(source, target))
                window.AddEdge(source, target);
        }

        private string FindClassName(uint functionID)
        {
            return engine.GetFunctionInfo(functionID).className;
        }

        SatelliteAssembliesResolver resolver;
        MainWindow window;
        RuntimeFlowExt.IEngine engine;
        System.Collections.Generic.Dictionary<uint, Thread> threads;
    }
}
